#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<getopt.h>

#include"mcd.h"
#include"util.h"
#include"word_buffer.h"
#include"json_parser.h"

typedef struct {
  int help;
  int verbose;
  int debug_mode;
  char *program_name;
  char *conll_filename;
  char *mcf_filename;
  char *mcd_filename;
  mcd *mcd_struct;
  char *json_filename;
} context;

void json2mcf_context_free(context *ctx)
{
  if(ctx){
    if(ctx->program_name)
      free(ctx->program_name);
    if(ctx->conll_filename)
      free(ctx->conll_filename);
    if(ctx->mcf_filename)
      free(ctx->mcf_filename);
    if(ctx->mcd_filename)
      free(ctx->mcd_filename);
    if(ctx->mcd_struct)
      mcd_free(ctx->mcd_struct);
    free(ctx);
  }
}

context *json2mcf_context_new(void)
{
  context *ctx = (context *)memalloc(sizeof(context));

  ctx->help = 0;
  ctx->verbose = 0;
  ctx->debug_mode = 0;
  ctx->program_name = NULL;
  ctx->conll_filename = NULL;
  ctx->mcf_filename = NULL;
  ctx->mcd_filename = NULL;
  ctx->mcd_struct = NULL;
  ctx->json_filename = NULL;
  return ctx;
}

void json2mcf_context_general_help_message(context *ctx)
{
  fprintf(stderr, "usage: %s [options]\n", ctx->program_name);
  fprintf(stderr, "Options:\n");
  fprintf(stderr, "\t-h --help             : print this message\n");
  fprintf(stderr, "\t-v --verbose          : activate verbose mode\n");
  fprintf(stderr, "\t-C --mcd              : mcd filename\n");
  fprintf(stderr, "\t-i --mcf              : mcf filename (read from stdin if absent)\n");
  fprintf(stderr, "\t-j --json             : json filename\n");
}

void json2mcf_check_options(context *ctx){
  if(ctx->help){
    json2mcf_context_general_help_message(ctx);
    exit(1);
  }
}

context *json2mcf_context_read_options(int argc, char *argv[])
{
  int c;
  int option_index = 0;
  context *ctx = json2mcf_context_new();

  ctx->program_name = strdup(argv[0]);

  static struct option long_options[7] =
    {
      {"help",                no_argument,       0, 'h'},
      {"verbose",             no_argument,       0, 'v'},
      {"debug",               no_argument,       0, 'd'},
      {"conll",               required_argument, 0, 'o'},
      {"mcd",                 required_argument, 0, 'C'}, 
      {"mcf",                 required_argument, 0, 'i'},
      {"json",                 required_argument, 0, 'j'},
    };
  optind = 0;
  opterr = 0;
  
  while ((c = getopt_long (argc, argv, "hvdo:C:i:j:", long_options, &option_index)) != -1){ 
    switch (c)
      {
      case 'd':
	ctx->debug_mode = 1;
	break;
      case 'h':
	ctx->help = 1;
	break;
      case 'v':
	ctx->verbose = 1;
	break;
      case 'o':
	ctx->conll_filename = strdup(optarg);
	break;
      case 'i':
	ctx->mcf_filename = strdup(optarg);
	break;
      case 'C':
	ctx->mcd_filename = strdup(optarg);
	break;
      case 'j':
	ctx->json_filename = strdup(optarg);
	break;
      }
  }

  if(ctx->mcd_filename){
    ctx->mcd_struct = mcd_read(ctx->mcd_filename, ctx->verbose);
  }
  else{
    ctx->mcd_struct = mcd_build_wpmlgfs();
  }

  return ctx;
}

void json2mcf_print_word_buffer(FILE *f, word_buffer *wb)
{
  int i;
  word *w;
  int col_nb = 0;
  mcd *mcd_struct = word_buffer_get_mcd(wb);
  char *string;
  int wf;
  
  for(i=0; i < wb->nbelem; i++){
    w = word_buffer_get_word_n(wb, i);
    for(col_nb=0; col_nb < mcd_struct->nb_col;col_nb++){
      wf = mcd_struct->wf[col_nb];
      if(col_nb > 0) {fprintf(f, "\t");}
      if(mcd_struct->representation[col_nb] == MCD_REPRESENTATION_INT){
	fprintf(f, "%d", w->wf_array[wf]);
      }
      if(mcd_struct->representation[col_nb] == MCD_REPRESENTATION_VOCAB){
	string = mcd_get_str(mcd_struct, w->wf_array[wf], col_nb);
	//	string = dico_int2string(mcd_struct->dico_array[col_nb], w->wf_array[wf]);
	if(string)
	  fprintf(f, "%s", string);
	else
	  fprintf(f, "_");
      }
    }
    fprintf(f, "\n");
  }
}
      
/*    if((mcd_get_gov_col(mcd_struct) == -1)
       && (mcd_get_label_col(mcd_struct) == -1)
       && (mcd_get_sent_seg_col(mcd_struct) == -1)){
      printf("%s\t", word_get_input(w));
      printf("%d\t", word_get_gov(w));
      label = (word_get_label(w) == -1)? NULL : dico_int2string(dico_labels, word_get_label(w));
      if(label != NULL)
	printf("%s\t", label) ;
      else
	printf("_\t");
      if(word_get_sent_seg(w) == 1)
	printf("1\n") ;
      else
	printf("0\n");
    }
    else{
      buffer = strdup(w->input);
      token = strtok(buffer, "\t");
      col_nb = 0;
      while(token){
	if(col_nb != 0) printf("\t");
	if(col_nb == mcd_get_gov_col(mcd_struct)){
	  printf("%d", word_get_gov(w));
	}
	else
	  if(col_nb == mcd_get_label_col(mcd_struct)){
	    label = (word_get_label(w) == -1)? NULL : dico_int2string(dico_labels, word_get_label(w));
	    if(label != NULL)
	      printf("%s", label) ;
	    else
	      printf("_");
	  }
	  else
	    if(col_nb == mcd_get_sent_seg_col(mcd_struct)){
	      if(word_get_sent_seg(w) == 1)
		printf("1") ;
	      else
		printf("0");
	    }
	    else{
	      word_print_col_n(stdout, w, col_nb);
	    }
	col_nb++;
	token = strtok(NULL, "\t");
      }
      if((col_nb <= mcd_get_gov_col(mcd_struct)) || (mcd_get_gov_col(mcd_struct) == -1)){
	printf("\t%d", word_get_gov(w));
      }
      if((col_nb <= mcd_get_label_col(mcd_struct)) || (mcd_get_label_col(mcd_struct) == -1)){
	label = (word_get_label(w) == -1)? NULL : dico_int2string(dico_labels, word_get_label(w));
	if(label != NULL)
	  printf("\t%s", label) ;
	else
	  printf("\t_");
      }
      if((col_nb <= mcd_get_sent_seg_col(mcd_struct)) || (mcd_get_sent_seg_col(mcd_struct) == -1)){
	if(word_get_sent_seg(w) == 1)
	  printf("\t1") ;
	else
	  printf("\t0");
      }
      printf("\n");
      free(buffer);
    }
  }
}

*/

void update_segment(word_buffer *wb, int start, int end, char *label, char *status_seg, char *status_lab, int offset)
{
  int index;
  word *w;
  int label_code = -1;
  dico *d;
  mcd *mcd_struct = NULL;

  mcd_struct = word_buffer_get_mcd(wb); 
  d = mcd_struct->dico_array[mcd_get_pos_col(mcd_struct)];
  
  if(status_lab && !strcmp(status_lab, "G")){
    fprintf(stderr, "updating label of segment [%d-%d] with \"%s\"\n", start, end, label);
    w = word_buffer_get_word_n(wb, offset + start);
    if(d)
      label_code = dico_string2int(d, label);
    if(label_code == -1)
      fprintf(stderr, "label %s unknown\n", label);
    else
      word_set_pos(w, label_code);
  }
}

void process_segment(json_attr_val *avl, word_buffer *wb, int offset)
{
  int start, end;
  char *label, *status_seg, *status_lab;
  json_attr_val *av;

  for(av = avl; av != NULL; av = av->next){
    //      printf("attr = %s\n", av->attr);
    if(!strcmp(av->attr, "start")){start = (int)(av->val->u.number); continue;}
    if(!strcmp(av->attr, "end")){end = (int)(av->val->u.number); continue;}
    if(!strcmp(av->attr, "label")){label = av->val->u.string; continue;}
    if(!strcmp(av->attr, "status_seg")){status_seg = av->val->u.string; continue;}
    if(!strcmp(av->attr, "status_lab")){status_lab = av->val->u.string; continue;}
  }
  update_segment(wb, start, end, label, status_seg, status_lab, offset);
  //   printf("segment : start = %d end = %d label = %s status_seg = %s status_lab = %s\n", start, end, label, status_seg, status_lab);

}

void process_segments(json_struct *segments, word_buffer *wb, int offset)
{
  json_struct *segment;
  //  printf("process_segments\n");
  for(segment = segments->u.first; segment != NULL; segment = segment->next){
    process_segment(segment->u.attr_val_list, wb, offset);
  }
}

// {"orig": 1, "dest":2, "label": "suj", "status_link": "", "status_lab": "", "timestamp": "", "author": "", "target": ""},

void update_link(word_buffer *wb,  int orig, int dest, char *label, char *status_link, char *status_lab, int offset)
{

  int index;
  word *w = NULL;
  int label_code = -1;
  dico *d;
  mcd *mcd_struct = NULL;

  mcd_struct = word_buffer_get_mcd(wb); 
  d = mcd_struct->dico_array[mcd_get_label_col(mcd_struct)];
  
  if(status_lab && !strcmp(status_lab, "G")){
    fprintf(stderr, "updating label of link %d -> %d with \"%s\"\n", orig, dest, label);
    w = word_buffer_get_word_n(wb, offset + orig);
    
    if(d)
      label_code = dico_string2int(d, label);
    if(label_code == -1)
      fprintf(stderr, "label %s unknown\n", label);
    else
      word_set_label(w, label_code);
  }
  
  if(status_link && !strcmp(status_link, "G")){
    fprintf(stderr, "updating governor of token %d with %d\n", orig, dest);
    w = word_buffer_get_word_n(wb, offset + orig);
    word_set_gov(w, dest - orig);
  }

  
}

void process_link(json_attr_val *avl, word_buffer *wb, int offset)
{
  int orig, dest;
  char *label, *status_link, *status_lab;
  json_attr_val *av;

  for(av = avl; av != NULL; av = av->next){
    //      printf("attr = %s\n", av->attr);
    if(!strcmp(av->attr, "orig")){orig = (int)(av->val->u.number); continue;}
    if(!strcmp(av->attr, "dest")){dest = (int)(av->val->u.number); continue;}
    if(!strcmp(av->attr, "label")){label = av->val->u.string; continue;}
    if(!strcmp(av->attr, "status_link")){status_link = av->val->u.string; continue;}
    if(!strcmp(av->attr, "status_lab")){status_lab = av->val->u.string; continue;}
  }
  //  fprintf(stderr, "link : orig = %d dest = %d label = %s status_link = %s status_lab = %s\n", orig, dest, label, status_link, status_lab);
  update_link(wb, orig, dest, label, status_link, status_lab, offset);


}

void process_links(json_struct *links, word_buffer *wb, int offset)
{
  json_struct *link;
  //  printf("process_links\n");
  for(link = links->u.first; link != NULL; link = link->next){
    process_link(link->u.attr_val_list, wb, offset);
  }

}

int get_id_of_first_token_in_document(json_struct *document)
{
  json_attr_val *avl = NULL;
  json_struct *tokens, *token;
  json_attr_val *avl2 = NULL;
  for(avl = document->u.attr_val_list; avl != NULL; avl = avl->next){
    if(!strcmp(avl->attr, (char *)"tokens")){
      tokens = avl->val;
      if(tokens){
	token = tokens->u.first;
	if(token){
	  for(avl2 = token->u.attr_val_list; avl2 != NULL; avl2 = avl2->next){
	    if(!strcmp(avl2->attr, (char *)"id"))
	      return (int)avl2->val->u.number;
	  }
	}
      }
    }
  }
  return -1;
}

void process_document(json_struct *document, word_buffer *wb)
{
  json_attr_val *avl = NULL;
  int offset = get_id_of_first_token_in_document(document);
  //  printf("process_document, offset = %d\n", offset);
  for(avl = document->u.attr_val_list; avl != NULL; avl = avl->next){
    //    if(!strcmp(avl->attr, (char *)"id")) printf("id = %s\n", avl->val->u.string);
    if(!strcmp(avl->attr, (char *)"segments")) process_segments(avl->val, wb, offset);
    if(!strcmp(avl->attr, (char *)"links")) process_links(avl->val, wb, offset);
  }
}

void process_documents(json_struct *documents, word_buffer *wb)
{
  json_struct *document;
  //  printf("process_documents\n");
  for(document = documents->u.first; document != NULL; document = document->next){
    process_document(document, wb);
  }
}

int main(int argc, char *argv[])
{
  FILE *output_file;
  context *ctx = json2mcf_context_read_options(argc, argv);
  word_buffer *wb = NULL;
  word *w = NULL;
  int first_sentence = 1;
  int new_sentence = 1;
  int index_first_word;
  int index_last_word;
  int sentence_nb = 0;
  json_struct *root = NULL;
  json_struct *document = NULL;
  json_attr_val *avl = NULL;

  json2mcf_check_options(ctx);
  mcd_extract_dico_from_corpus(ctx->mcd_struct, ctx->mcf_filename);
  wb = word_buffer_load_mcf(ctx->mcf_filename, ctx->mcd_struct);
  root = json_parse_full(ctx->json_filename, 0);
  
  if(root->type != JSON_AVL){
    fprintf(stderr, "erreur le json doit être un objet\n");
    exit(1);
  }
  for(avl = root->u.attr_val_list; avl != NULL; avl = avl->next){
    //    printf("section %s\n", avl->attr);
    if(!strcmp(avl->attr, (char *)"documents")){
      process_documents(avl->val, wb);
    }
  }
  
  //json_print_struct(stdout, root);
  json2mcf_print_word_buffer(stdout, wb);
  json_free_struct(root);
  json2mcf_context_free(ctx);
  return 0;
}
